<?php

/**
 * 20/10/2015
 * @author Victor Novy
 */

namespace Modules\DF\Services;

use \Teknisa\Libs\Util\DateUtil;
use \Doctrine\ORM\EntityManager;
use \Zeedhi\Framework\DB\StoredProcedure\Param;
use \Zeedhi\Framework\DB\StoredProcedure\StoredProcedure;
use \Estoque\Entities\Lanctoestoq;
use \Estoque\Entities\Movilote;
use \Estoque\Entities\Posestdia;
use \Estoque\Entities\Itlanctoest;
use \Geral\Entities\Produto;
use \Zeedhi\Framework\ORM\DateTime;

class ValidacoesEstoqueService {
    const ENTRADA       = 'E';
    const SAIDA         = 'S';
    const TIPO_CUSTO_PADRAO = '03';
    const ESTOQUE_VAZIO = ' ';

    protected $entityManager;
    private $lanctoestoq;
    protected $errors = array();
    
    public function __construct (EntityManager $entityManager) {
        $this->entityManager = $entityManager;
    }
    
    /* ******************** Consulta o Custo e a Quantidade do Produto ******************** */
    //$filial - cdfilimovi
    //$produto - cdprodesto
    public function consultaCustoQuantidadeProduto(
        $filial, $produto, $dtLancamento, $tipoCusto = self::TIPO_CUSTO_PADRAO,
        $almoxarifado = self::ESTOQUE_VAZIO, $localEstoque = self::ESTOQUE_VAZIO,
        $lote = self::ESTOQUE_VAZIO, $subLote = self::ESTOQUE_VAZIO , $operacao
    ){
        $custoQuantidade = null;
        $operacao = $this->getTipoOperac($operacao);
        $paramFilial = $this->getParamFilial($filial);
        
        $cdtipcustest = $paramFilial["CDTIPCUSTEST"];
        $tpCustoOp = $operacao["CDTIPOCUSTO"];
        $cdprecobase = $operacao["CDPRECOBASE"];

        if(!empty($tpCustoOp)){
            $tipoCusto = $tpCustoOp;
        }else if(!empty($cdtipcustest)){
            $tipoCusto = $cdtipcustest;
        }else{
            $custoQuantidade = array('VRCUSTOPROD' => '0', "ExistsVr" => false, "ExistsTp" => false);
            return $custoQuantidade;
        }
        
        if(!empty($cdprecobase)){
            $custoQuantidade = $this->consultaCustoBase($produto, $cdprecobase);
        }else{
              $custoQuantidade = $this->consultaCustoQuant(
                $filial, $tipoCusto, $produto, $almoxarifado, $localEstoque, $lote, $subLote, $dtLancamento
            ); 
        }
        
        if (!$custoQuantidade){
            $custoQuantidade = array('VRCUSTOPROD' => '0', "ExistsVr" => false, "ExistsTp" => true);
            return $custoQuantidade;
        }

        return array('VRCUSTOPROD' => $custoQuantidade['VRCUSTOPROD'], "ExistsVr" => true, "ExistsTp" => true);
    }
    
    public function getParamFilial($cdfilial){
        $sql = <<<SQL
        SELECT * FROM PARAMFILIAL WHERE CDFILIAL = :CDFILIAL
SQL;

        $params = array(
            "CDFILIAL" => $cdfilial
        );

        try{
            return $this->entityManager->getConnection()->fetchAssoc($sql, $params);
        } catch(\Exception $e) {
            throw $e;
        }
        
    }
    
    public function getTipoOperac($operacao){
        $sql = <<<SQL
        SELECT * FROM TIPOOPERAC WHERE IDENTRSAIDOP = 'S' AND CDTIPOOPER = :CDTIPOOPER
SQL;

        $params = array(
            "CDTIPOOPER" => $operacao
        );

        try{
            return $this->entityManager->getConnection()->fetchAssoc($sql, $params);
        } catch(\Exception $e) {
            throw $e;
        }
    }
    
    public function consultaCustoQuant(
        $filial, $tipoCusto, $produto, $almoxarifado, $localEstoque, $lote, $subLote, $dtLancamento
    )
    {
        $sql = <<<SQL
        SELECT
            NVL(CUSTO_PRODUTO_LCEST(:P_CDFILIAL, D.CDPRODUTO, :P_INTPCUSTO, (TO_DATE(:P_DTLANC) - 1), SYSDATE,
                                    D.CDALMOXARIFE, D.CDLOCALESTOQ, D.NRLOTEESTQ, D.NRSUBLOTE), 0) * NVL(P.VRFATOCONV, 1) AS VRCUSTOPROD
        FROM
            PRODUTO     P,
            POSESTDIA   D,
            LOTE        L
        WHERE
            P.CDPRODUTO = :P_CDPRODUTO
            AND D.CDFILIAL = :P_CDFILIAL
            AND D.CDALMOXARIFE = :P_CDALMOXARIFE
            AND D.CDLOCALESTOQ = :P_CDLOCALESTOQ
            AND D.NRLOTEESTQ = :P_NRLOTEESTQ
            AND D.NRSUBLOTE = :P_NRSUBLOTE
            AND D.CDPRODUTO = P.CDPRODESTO
            AND ( D.QTESTOQDIA > :P_QTDE
                  OR :P_QTDE = - 1 )
            AND D.CDFILIAL = L.CDFILIAL
            AND D.NRLOTEESTQ = L.NRLOTEESTQ
            AND D.NRSUBLOTE = L.NRSUBLOTE
            AND D.CDPRODUTO = L.CDPRODUTO
            AND D.DTPOSIESTQ = (SELECT MAX(B.DTPOSIESTQ)
                                 FROM POSESTDIA B
                                WHERE B.CDFILIAL     = D.CDFILIAL
                                  AND B.CDPRODUTO    = D.CDPRODUTO
                                  AND B.CDALMOXARIFE = D.CDALMOXARIFE
                                  AND B.CDLOCALESTOQ = D.CDLOCALESTOQ
                                  AND B.NRLOTEESTQ   = D.NRLOTEESTQ
                                  AND B.NRSUBLOTE    = D.NRSUBLOTE
                                  AND B.DTPOSIESTQ   <= TO_DATE(:P_DTLANC) + 1)
SQL;

        $params = array(
            "P_CDFILIAL" => $filial,
            "P_INTPCUSTO" => $tipoCusto,
            "P_CDALMOXARIFE" => $almoxarifado,
            "P_CDLOCALESTOQ" => $localEstoque,
            "P_CDPRODUTO" => $produto,
            "P_NRLOTEESTQ" => $lote,
            "P_NRSUBLOTE" => $subLote,
            "P_DTLANC" => $dtLancamento,
            ":P_QTDE" => "-1"
        );

        try{
            return $this->entityManager->getConnection()->fetchAssoc($sql, $params);
        } catch(\Exception $e) {
            throw $e;
        }
    }
    
    public function consultaCustoBase($produto, $cdprecobase){
        $sql = <<<SQL
        SELECT
                I.VRITEPREBASE AS  VRCUSTOPROD
            FROM
                ITEMPREBASE I
            WHERE
                (I.CDPRECOBASE = :P_CDPRECOBASE) AND
                (I.CDPRODUTO = :P_CDPRODUTO)
SQL;
        
        $params = array(
            "P_CDPRODUTO" => $produto,
            "P_CDPRECOBASE" => $cdprecobase
        );
    
        try{
            return $this->entityManager->getConnection()->fetchAssoc($sql, $params);
        } catch(\Exception $e) {
            throw $e;
        }
    }
}